#!/bin/bash


###########################################################


export QCG_RMI_PORT=@@@RMI_PORT@@@

export QCG_LOCATION=@@@SHARE_DIR@@@/broker

export QCG_PID_FILE=@@@VAR_DIR@@@/qcg-broker.pid

export QCG_LOG_DIR=@@@LOG_DIR@@@


###########################################################


function qcgUtilMuteOut {
	exec 6<&1
	exec 7<&2
	exec > /dev/null 2>&1
}


function qcgUtilUnmuteOut {
	exec 1<&6 6<&-
	exec 2<&7 7<&-
}


function qcgUtilCheckPort {
	local cport=$1
	local loops=$2
	local sltime=$3

	local code=1
	local loop=0

#	echo "checking [$cport] port with [$loops] loops and [$sltime] between each loop"
	qcgUtilMuteOut

	while [ "$loop" -lt "$loops" ]; do
		loop=`expr $loop + 1`

		echo "HEAD" > /dev/tcp/127.0.0.1/$cport
		code=$?
		echo "code [$code]"
		if [ "$code" -eq "0" ]; then
			break
		fi

		if [ "$loop" -lt "$loops" ]; then
			sleep $sltime
		fi
	done
 
	qcgUtilUnmuteOut

	echo $code
}


###########################################################


function qcgBrokerStartup {
	if [ "$1X" == "-forceX" ]; then
	    rm $QCG_PID_FILE
	fi

	if [ -f "$QCG_PID_FILE" ]; then
		echo "QCG already started or PID file \"$QCG_PID_FILE\" not removed"
		exit 1
	fi

	if [ "${QCG_LOCATION}" = "" ]
	then
	    echo Environment variable QCG_LOCATION is not set
	    QCG_LOCATION='.'
	fi

#	echo QCG_LOCATION=${QCG_LOCATION}
	for i in ${QCG_LOCATION}/jars/axis/*.jar;do CLASSPATH=$i:$CLASSPATH; done
	for i in ${QCG_LOCATION}/jars/castor/*.jar;do CLASSPATH=$i:$CLASSPATH; done
	for i in ${QCG_LOCATION}/jars/jakarta/*.jar;do CLASSPATH=$i:$CLASSPATH; done
	for i in ${QCG_LOCATION}/jars/jargon/*.jar;do CLASSPATH=$i:$CLASSPATH; done
	for i in ${QCG_LOCATION}/jars/jglobus/*.jar;do CLASSPATH=$i:$CLASSPATH; done
	for i in ${QCG_LOCATION}/jars/msv/*.jar;do CLASSPATH=$i:$CLASSPATH; done
	for i in ${QCG_LOCATION}/jars/schedframe/*.jar;do CLASSPATH=$i:$CLASSPATH; done
	for i in ${QCG_LOCATION}/jars/smoa/*.jar;do CLASSPATH=$i:$CLASSPATH; done
	for i in ${QCG_LOCATION}/jars/unicore/*.jar;do CLASSPATH=$i:$CLASSPATH; done
	for i in ${QCG_LOCATION}/jars/xerces/*.jar;do CLASSPATH=$i:$CLASSPATH; done
	local CLASSPATH=${QCG_LOCATION}/jars:${QCG_LOCATION}/jars/castor:$CLASSPATH

	CLASSPATH=$CLASSPATH:${QCG_LOCATION}/lib/qcg-broker.jar
	CLASSPATH=$CLASSPATH:${QCG_LOCATION}/lib/qcg-broker_schemas.jar
	CLASSPATH=$CLASSPATH:${QCG_LOCATION}/lib/qcg-broker_services.jar
	CLASSPATH=$CLASSPATH:${QCG_LOCATION}/lib/qcg-broker_stubs.jar
	CLASSPATH=$CLASSPATH:${QCG_LOCATION}/lib/qcg-broker_webservices.jar
	CLASSPATH=$CLASSPATH:${QCG_LOCATION}/lib/qcg-broker_schedplugin.jar
	CLASSPATH=$CLASSPATH:${QCG_LOCATION}/lib/qcg-broker_api.jar

	local OPTS="-DQCG_LOCATION=${QCG_LOCATION}"
	OPTS="$OPTS -Dgrms.broker.root=${QCG_LOCATION}"
	OPTS="$OPTS -Dsun.lang.ClassLoader.allowArraySyntax=true"
	OPTS="$OPTS -Xms512m -Xmx1024m"
	OPTS="$OPTS -Dorg.globus.ftp.openTO=30000"
	OPTS="$OPTS -Dorg.globus.tcp.port.range=20000,21000"

	# Unicore
	OPTS="$OPTS -Djavax.xml.stream.XMLEventFactory=com.ctc.wstx.stax.WstxEventFactory"
	OPTS="$OPTS -Djavax.xml.stream.XMLInputFactory=com.ctc.wstx.stax.WstxInputFactory"
	OPTS="$OPTS -Djavax.xml.stream.XMLOutputFactory=com.ctc.wstx.stax.WstxOutputFactory"

	java ${OPTS} -cp ${CLASSPATH}:. org.qcg.broker.server.FactoryServer \
		> @@@LOG_DIR@@@/broker-stdouterr 2>&1 &
	local result=`qcgUtilCheckPort $QCG_RMI_PORT 150 2s`

	if [ "$result" -eq "0" ]; then
		echo $! > $QCG_PID_FILE
		PID=`cat $QCG_PID_FILE`
		echo "QCG PSNC started with pid [$PID]"
		return 0
	else
		echo "Failed to start QCG-Broker - check logs for detail description"
		return 1
	fi
}


function qcgBrokerShutdown {
	if [ "${QCG_LOCATION}" = "" ]; then
	    echo Environment variable QCG_LOCATION is not set
	    QCG_LOCATION='.'
	fi

	local result=`qcgUtilCheckPort $QCG_RMI_PORT 1 2s`

	if [ "$result" -ne "0" ]; then
		echo "QCG-Broker isn't running"
		return 1
	fi

	if [ ! -f "$QCG_PID_FILE" ]; then
		echo "PID file \"$QCG_PID_FILE\" not found"
		exit 1
	fi

	local PID=`cat $QCG_PID_FILE`
	echo "trying to stop QCG-Broker with PID [$PID]"
	kill -s SIGTERM $PID

	if [ $? -ne 0 ]; then
		echo "failed to stop QCG-Broker"
	else
		sleep 5
		kill -9 $PID &> /dev/null
		echo "QCG-Broker stopped"
	fi

	rm -f $QCG_PID_FILE
	return 0
}


function qcgBrokerStatus {
	local result=`qcgUtilCheckPort $QCG_RMI_PORT 1 2s`

	if [ "$result" -eq "0" ]; then
		echo "QCG-Broker is running"
		return 0
	else
		echo "QCG-Broker isn't running"
		return 1
	fi
}


###########################################################
