<?xml version="1.0" encoding="UTF-8"?>

<xsl:stylesheet version="1.0" 
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:jsdl="http://schemas.ggf.org/jsdl/2005/11/jsdl" 
	xmlns:jsdl-posix="http://schemas.ggf.org/jsdl/2005/11/jsdl-posix"
	xmlns:jsdl-hpcpa="http://schemas.ggf.org/jsdl/2006/07/jsdl-hpcpa"
	xmlns:msxsl="urn:schemas-microsoft-com:xslt" 
	xmlns:xalan="http://xml.apache.org/xalan"
	xmlns:qcg="http://qcg.psnc.pl/jsdlext/2011/01/qcg-allocation"
	extension-element-prefixes="msxsl">

	<xsl:output method="xml" version="1.0" encoding="UTF-8" indent="yes"/>

	<xsl:variable name="FILE_URL_PREFIX">file:///</xsl:variable>	
	
	<xsl:key name="FILESYSTEMS" match="/jsdl:JobDefinition/jsdl:JobDescription/jsdl:Resources/jsdl:FileSystem" use="@name"/>
	
	<xsl:template match="/">		
		<xsl:apply-templates select="jsdl:JobDefinition/jsdl:JobDescription"/>		
	</xsl:template>
	
	<xsl:template match="jsdl:JobDescription">
		<xsl:element name="qcgJob">

			<xsl:attribute name="appId">
				<xsl:value-of select="jsdl:JobIdentification/jsdl:JobName"/>
			</xsl:attribute>
			
			<xsl:if test="jsdl:JobIdentification/jsdl:JobProject">
				<xsl:attribute name="project">
					<xsl:value-of select="jsdl:JobIdentification/jsdl:JobProject[1]"/>
				</xsl:attribute>
			</xsl:if>
		
		<xsl:apply-templates select="jsdl:JobIdentification/jsdl:Description"/>
		
		<xsl:call-template name="TASK_TEMPLATE"/>
			
		</xsl:element>
	</xsl:template>
		
	<xsl:template match="jsdl:Description">
		<xsl:element name="description">
			<xsl:value-of select="."/>
		</xsl:element>
	</xsl:template>	
		
	<xsl:template name="TASK_TEMPLATE">
		<xsl:element name="task">
		
			<xsl:attribute name="taskId">
				<xsl:choose>
					<xsl:when test="jsdl:JobIdentification/jsdl:JobName">
						<xsl:value-of select="jsdl:JobIdentification/jsdl:JobName"/>
					</xsl:when>
					<xsl:otherwise>task</xsl:otherwise>
				</xsl:choose>
			</xsl:attribute>
			
			<xsl:if test="count(jsdl:DataStaging/jsdl:DeleteOnTermination[text() = 'false']) &gt; 0">
					<xsl:attribute name="persistent">true</xsl:attribute>
			</xsl:if>
			
			<xsl:apply-templates select="jsdl:Resources/jsdl:CandidateHosts"/>
			
			<xsl:call-template name="REQUIREMENTS_TEMPLATE"/>
			
			<xsl:call-template name="EXECUTION_TEMPLATE"/>
			
			<xsl:call-template name="EXECUTION_TIME_TEMPLATE"/>
			
		</xsl:element>
	</xsl:template>
	
	<xsl:template name="EXECUTION_TEMPLATE">
		<xsl:element name="execution">
		
			<xsl:attribute name="type">
				<xsl:choose>
					<xsl:when test="jsdl:Resources/jsdl:TotalCPUCount">mpi</xsl:when>
					<xsl:otherwise>single</xsl:otherwise>
			</xsl:choose>

			</xsl:attribute>
			
			<xsl:call-template name="EXECUTABLE_TEMPLATE"/>
			<xsl:call-template name="ARGUMENTS_TEMPLATE"/>
			<xsl:call-template name="STREAMS_TEMPLATE"/>
			<xsl:call-template name="DATA_STAGING_TEMPLATE"/>
			<xsl:call-template name="ENVIRONMENT_TEMPLATE"/>
			
		</xsl:element>
	</xsl:template>
	
	<xsl:template name="EXECUTABLE_TEMPLATE">
		<xsl:element name="executable">
				<xsl:choose>
					<xsl:when test="jsdl:Application/jsdl:ApplicationName">
						<xsl:element name="application">
							<xsl:attribute name="name">
								<xsl:value-of select="jsdl:Application/jsdl:ApplicationName"/>
							</xsl:attribute>
							
							<xsl:if test="jsdl:Application/jsdl:ApplicationVersion">
								<xsl:attribute name="version">
									<xsl:value-of select="jsdl:Application/jsdl:ApplicationVersion"/>
								</xsl:attribute>
							</xsl:if>
						</xsl:element>
					</xsl:when>
					<xsl:when test="jsdl:Application/jsdl-hpcpa:HPCProfileApplication/jsdl-hpcpa:Executable">
						<xsl:element name="execFile">
							<xsl:element name="file">
								<xsl:element name="location">
									<xsl:attribute name="type">URL</xsl:attribute>
									<xsl:value-of select="jsdl:Application/jsdl-hpcpa:HPCProfileApplication/jsdl-hpcpa:Executable"/>												</xsl:element>
							</xsl:element>
						</xsl:element>
					</xsl:when>
					<xsl:otherwise>
						<xsl:message terminate="yes">NEITHER APPLICATION NOR EXEC_FILE</xsl:message>
					</xsl:otherwise>
				</xsl:choose>
		</xsl:element>
	</xsl:template>
	
	<xsl:template name="ARGUMENTS_TEMPLATE">
		<xsl:if test="jsdl:Application/jsdl-hpcpa:HPCProfileApplication/jsdl-hpcpa:Argument">
			<xsl:element name="arguments">
				<xsl:apply-templates select="jsdl:Application/jsdl-hpcpa:HPCProfileApplication/jsdl-hpcpa:Argument"/>
			</xsl:element>
		</xsl:if>
	</xsl:template>
	
	<xsl:template match="jsdl-hpcpa:Argument">
		<xsl:element name="value">
			<xsl:value-of select="."/>
		</xsl:element>
	</xsl:template>
	
	<xsl:template match="jsdl:CandidateHosts">
		<xsl:element name="candidateHosts">
			<xsl:for-each select="jsdl:HostName">
				<xsl:element name="hostName">
					<xsl:value-of select="."/>
				</xsl:element>
			</xsl:for-each>
		</xsl:element>
	</xsl:template>
	
	<xsl:template name="STREAMS_TEMPLATE">
		<xsl:apply-templates select="jsdl-hpcpa:HPCProfileApplication/jsdl-hpcpa:Input"/>
		<xsl:apply-templates select="jsdl:Application/jsdl-hpcpa:HPCProfileApplication/jsdl-hpcpa:Output"/>
		<xsl:apply-templates select="jsdl:Application/jsdl-hpcpa:HPCProfileApplication/jsdl-hpcpa:Error"/>
	</xsl:template>
	
	<xsl:template match="jsdl-hpcpa:Input">
		<xsl:element name="stdin">
			<xsl:element name="location">
				<xsl:attribute name="type">URL</xsl:attribute>
				<xsl:value-of select="."/>
			</xsl:element>
		</xsl:element>	
	</xsl:template>

       <xsl:template match="jsdl-hpcpa:Output">
		<xsl:call-template name="STD_STREAM_TEMPLATE">
			<xsl:with-param name="NAME">stdout</xsl:with-param>
			<xsl:with-param name="VALUE" select="."/>
		</xsl:call-template>	
	</xsl:template>

      <xsl:template match="jsdl-hpcpa:Error">
		<xsl:call-template name="STD_STREAM_TEMPLATE">
			<xsl:with-param name="NAME">stderr</xsl:with-param>
			<xsl:with-param name="VALUE" select="."/>
		</xsl:call-template>	
	</xsl:template>
	
	<xsl:template name="STD_STREAM_TEMPLATE">
		<xsl:param name="NAME"/>
		<xsl:param name="VALUE"/>
		
		<xsl:element name="{$NAME}">
			<xsl:element name="file">
				<xsl:element name="location">
					<xsl:attribute name="type">URL</xsl:attribute>
					<xsl:value-of select="$VALUE"/>
				</xsl:element>
			</xsl:element>
		</xsl:element>
	</xsl:template>
	
	<xsl:template name="ENVIRONMENT_TEMPLATE">
		<xsl:if test="jsdl:Application/jsdl-hpcpa:HPCProfileApplication/jsdl-hpcpa:Environment">
			<xsl:element name="environment">
				<xsl:apply-templates select="jsdl:Application/jsdl-hpcpa:HPCProfileApplication/jsdl-hpcpa:Environment"/>
			</xsl:element>
		</xsl:if>
	</xsl:template>
	
	<xsl:template match="jsdl-hpcpa:Environment">
		<xsl:element name="variable">
			<xsl:attribute name="name">
				<xsl:value-of select="@name"/>
			</xsl:attribute>
			<xsl:value-of select="."/>
		</xsl:element>
	</xsl:template>
	
	<xsl:template name="REQUIREMENTS_TEMPLATE">
		<xsl:if test="jsdl:Application/jsdl:ApplicationName or jsdl:Resources">
				<xsl:element name="requirements">
							
					<xsl:if test="jsdl:Application/jsdl:ApplicationName or
	                                 jsdl:Resources/jsdl:IndividualCPUSpeed or
				             jsdl:Resources/jsdl:IndividualCPUCount or
				             jsdl:Resources/jsdl:IndividualNetworkBandwidth or
				             jsdl:Resources/jsdl:IndividualPhysicalMemory or
				             jsdl:Resources/jsdl:IndividualDiskSpace or
				             jsdl:Resources/jsdl:CPUArchitecture or
				             jsdl:Resources/jsdl:OperatingSystem/jsdl:OperatingSystemType/jsdl:OperatingSystemName or
					      jsdl:Resources/jsdl:OperatingSystem/jsdl:OperatingSystemVersion">
					      		<xsl:element name="resourceRequirements">
								<xsl:element name="computingResource">
								       <xsl:apply-templates select="jsdl:Application/jsdl:ApplicationName">
										<xsl:with-param name="VERSION">
											<xsl:value-of select="jsdl:Application/jsdl:ApplicationVersion"/>
										</xsl:with-param>
									</xsl:apply-templates>
									<xsl:apply-templates select="jsdl:Resources/jsdl:CPUArchitecture/jsdl:CPUArchitectureName"/>
									<xsl:apply-templates select="jsdl:Resources/jsdl:IndividualCPUSpeed"/>
									<xsl:apply-templates select="jsdl:Resources/jsdl:IndividualCPUCount"/>
									<xsl:apply-templates select="jsdl:Resources/jsdl:IndividualNetworkBandwidth"/>
									<xsl:apply-templates select="jsdl:Resources/jsdl:IndividualPhysicalMemory"/>
									<xsl:apply-templates select="jsdl:Resources/jsdl:IndividualDiskSpace"/>
									<xsl:apply-templates select="jsdl:Resources/jsdl:OperatingSystem/jsdl:OperatingSystemType"/>
						                    <xsl:apply-templates select="jsdl:Resources/jsdl:OperatingSystem/jsdl:OperatingSystemVersion"/>
								</xsl:element>
							</xsl:element>
					</xsl:if>
					
					<xsl:if test="jsdl:Resources/jsdl:TotalCPUCount">
						<xsl:element name="topology">
							<xsl:element name="processes">
								<xsl:apply-templates select="jsdl:Resources/jsdl:TotalCPUCount"/>
							</xsl:element>
						</xsl:element>
					</xsl:if>
			</xsl:element>
		</xsl:if>
	</xsl:template>
	
	<xsl:template match="jsdl:TotalCPUCount">
		<xsl:element name="processesCount">
			<xsl:choose>
				<xsl:when test="jsdl:LowerBoundedRange and jsdl:UpperBoundedRange">
					<xsl:element name="range">
						<xsl:apply-templates select="jsdl:LowerBoundedRange"/>
						<xsl:apply-templates select="jsdl:UpperBoundedRange"/>
					</xsl:element>
				</xsl:when>
				<xsl:otherwise>
					<xsl:apply-templates select="jsdl:LowerBoundedRange"/>
					<xsl:apply-templates select="jsdl:UpperBoundedRange"/>
				</xsl:otherwise>
			</xsl:choose>
			<xsl:apply-templates select="jsdl:Exact"/>
			<xsl:apply-templates select="jsdl:Range"/>
		</xsl:element>
	</xsl:template>
	
	<xsl:template match="jsdl:ApplicationName">
		<xsl:param name="VERSION"/>
		<xsl:call-template name="HOST_PARAMETER_STRING_TEMPLATE">
			<xsl:with-param name="NAME">application</xsl:with-param>
			<xsl:with-param name="VALUES" select="."/>
			<xsl:with-param name="PROP_NAME">version</xsl:with-param>
			<xsl:with-param name="PROP_VALUE"><xsl:value-of select="$VERSION"/></xsl:with-param>
		</xsl:call-template>
	</xsl:template>
		
	<xsl:template match="jsdl:OperatingSystemName">
		<xsl:call-template name="HOST_PARAMETER_STRING_TEMPLATE">
			<xsl:with-param name="NAME">ostype</xsl:with-param>
			<xsl:with-param name="VALUES" select="."/>
		</xsl:call-template>
	</xsl:template>

	<xsl:template match="jsdl:OperatingSystemVersion">
		<xsl:call-template name="HOST_PARAMETER_STRING_TEMPLATE">
			<xsl:with-param name="NAME">osversion</xsl:with-param>
			<xsl:with-param name="VALUES" select="."/>
		</xsl:call-template>
	</xsl:template>
	
	<xsl:template match="jsdl:CPUArchitectureName">
		<xsl:call-template name="HOST_PARAMETER_STRING_TEMPLATE">
			<xsl:with-param name="NAME">cpuarch</xsl:with-param>
			<xsl:with-param name="VALUES" select="."/>
		</xsl:call-template>
	</xsl:template>

	<xsl:template match="jsdl:IndividualCPUSpeed">
		<xsl:call-template name="HOST_PARAMETER_VALUE_TEMPLATE">
			<xsl:with-param name="NAME">cpuspeed</xsl:with-param>
			<xsl:with-param name="VALUES" select="*"/>
			<xsl:with-param name="FACTOR">0.001</xsl:with-param>
		</xsl:call-template>
	</xsl:template>
	
	<xsl:template match="jsdl:IndividualCPUCount">
		<xsl:call-template name="HOST_PARAMETER_VALUE_TEMPLATE">
			<xsl:with-param name="NAME">cpucount</xsl:with-param>
			<xsl:with-param name="VALUES" select="*"/>
			<xsl:with-param name="FACTOR">1</xsl:with-param>
		</xsl:call-template>
	</xsl:template>
	
	<xsl:template match="jsdl:IndividualNetworkBandwidth">
		<xsl:call-template name="HOST_PARAMETER_VALUE_TEMPLATE">
			<xsl:with-param name="NAME">bandwidth</xsl:with-param>
			<xsl:with-param name="VALUES" select="*"/>
			<xsl:with-param name="FACTOR">
				<xsl:value-of select="1 div (8 * 1024)"/>
			</xsl:with-param>
		</xsl:call-template>	
	</xsl:template>
	
	<xsl:template match="jsdl:IndividualPhysicalMemory">
		<xsl:call-template name="HOST_PARAMETER_VALUE_TEMPLATE">
			<xsl:with-param name="NAME">memory</xsl:with-param>
			<xsl:with-param name="VALUES" select="*"/>
			<xsl:with-param name="FACTOR">
				<xsl:value-of select="1 div 1024"/>
			</xsl:with-param>
		</xsl:call-template>	
	</xsl:template>
	
	<xsl:template match="jsdl:IndividualDiskSpace">
		<xsl:call-template name="HOST_PARAMETER_VALUE_TEMPLATE">
			<xsl:with-param name="NAME">diskspace</xsl:with-param>
			<xsl:with-param name="VALUES" select="*"/>
			<xsl:with-param name="FACTOR">1</xsl:with-param>
		</xsl:call-template>	
	</xsl:template>

     <xsl:template name="HOST_PARAMETER_STRING_TEMPLATE">
		<xsl:param name="NAME"/>
		<xsl:param name="VALUES"/>
		<xsl:param name="PROP_NAME"/>
		<xsl:param name="PROP_VALUE"/>
		<xsl:element name="hostParameter">
			<xsl:attribute name="name">
				<xsl:value-of select="$NAME"/>
			</xsl:attribute>
			<xsl:element name="stringValue">
				<xsl:attribute name="value">
					<xsl:value-of select="$VALUES"/>
				</xsl:attribute>
				<xsl:if test="$PROP_NAME and $PROP_VALUE">
					<xsl:element name="property">
						<xsl:attribute name="name">
							<xsl:value-of select="$PROP_NAME"/>
						</xsl:attribute>
						<xsl:if test="$PROP_VALUE">
							<xsl:element name="value">
								<xsl:value-of select="$PROP_VALUE"/>
							</xsl:element>
						</xsl:if>
					</xsl:element>
				</xsl:if>
			</xsl:element>
		</xsl:element>
	</xsl:template>

	
	<xsl:template name="HOST_PARAMETER_VALUE_TEMPLATE">
		<xsl:param name="NAME"/>
		<xsl:param name="VALUES"/>
		<xsl:param name="FACTOR"/>
		<xsl:element name="hostParameter">
			<xsl:attribute name="name">
				<xsl:value-of select="$NAME"/>
			</xsl:attribute>
		<xsl:apply-templates select="xalan:nodeset($VALUES)">
				<xsl:with-param name="FACTOR" select="$FACTOR"/>
			</xsl:apply-templates>
		</xsl:element>
	</xsl:template>
	
	<xsl:template match="jsdl:UpperBoundedRange">
		<xsl:param name="FACTOR" select="1"/>
		<xsl:element name="max">
			<xsl:value-of select=". * $FACTOR"/>
		</xsl:element>
	</xsl:template>

	<xsl:template match="jsdl:LowerBoundedRange">
		<xsl:param name="FACTOR" select="1"/>
		<xsl:element name="min">
			<xsl:value-of select=". * $FACTOR"/>
		</xsl:element>
	</xsl:template>

	<xsl:template match="jsdl:Exact">
		<xsl:param name="FACTOR" select="1"/>
		<xsl:element name="value">
			<xsl:value-of select=". * $FACTOR"/>
		</xsl:element>
	</xsl:template>

	<xsl:template match="jsdl:Range">
		<xsl:param name="FACTOR" select="1"/>
		<xsl:element name="range">
			<xsl:element name="min">
				<xsl:value-of select="jsdl:LowerBound * $FACTOR"/>
			</xsl:element>
			<xsl:element name="max">
				<xsl:value-of select="jsdl:UpperBound * $FACTOR"/>
			</xsl:element>
		</xsl:element>
	</xsl:template>
		
	<xsl:template match="jsdl-posix:Executable">
		<xsl:element name="execfile">
			<xsl:attribute name="name">
				<xsl:call-template name="GET_FILE_NAME">
					<xsl:with-param name="FILE_PATH">
						<xsl:value-of select="."/>
					</xsl:with-param>
				</xsl:call-template>
			</xsl:attribute>
			<xsl:element name="url">
				<xsl:call-template name="FILESYSTEM_VALUE">
					<xsl:with-param name="FILESYSTEM_NAME">
						<xsl:value-of select="@filesystemName"/>
					</xsl:with-param>
					<xsl:with-param name="VALUE">
						<xsl:value-of select="."/>
					</xsl:with-param>
				</xsl:call-template>
			</xsl:element>
		</xsl:element>
	</xsl:template>
	
	<xsl:template match="jsdl-posix:Argument">
		<xsl:element name="value">
			<xsl:call-template name="FILESYSTEM_VALUE">
				<xsl:with-param name="FILESYSTEM_NAME">
					<xsl:value-of select="@filesystemName"/>
				</xsl:with-param>
				<xsl:with-param name="VALUE">
					<xsl:value-of select="."/>
				</xsl:with-param>
			</xsl:call-template>
		</xsl:element>
	</xsl:template>
	
	<xsl:template name="DATA_STAGING_TEMPLATE">			
		<xsl:if test="jsdl:DataStaging">
			<xsl:element name="stageInOut">
				<xsl:apply-templates select="jsdl:DataStaging"/>
			</xsl:element>	
		</xsl:if>
	</xsl:template>
	

	<xsl:template match="jsdl:DataStaging">
	   		
		<xsl:if test="jsdl:Source">
			<xsl:apply-templates select="jsdl:Source" mode="FILE_INOUT"/>
		</xsl:if>

             	<xsl:if test="jsdl:Target">
			<xsl:apply-templates select="jsdl:Target" mode="FILE_INOUT"/>
		</xsl:if>
	</xsl:template>


	<xsl:template match="*" mode="FILE_INOUT">
		  
		<xsl:element name="file">
			<xsl:attribute name="name">
				<xsl:call-template name="FILESYSTEM_VALUE">
					<xsl:with-param name="FILESYSTEM_NAME">
						<xsl:value-of select="../jsdl:FilesystemName"/>
					</xsl:with-param>
					<xsl:with-param name="VALUE">
						<xsl:value-of select="../jsdl:FileName"/>
					</xsl:with-param>
				</xsl:call-template>
			</xsl:attribute>
			
			<xsl:attribute name="type">
				<xsl:choose>
					<xsl:when test="name() = 'Source'">in</xsl:when>
					<xsl:when test="name() = 'Target'">out</xsl:when>
					<xsl:otherwise>
						<xsl:message terminate="no">
							Source or Target URI is missing
						</xsl:message>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:attribute>
			
			<xsl:if test="../jsdl:CreationFlag = 'append'">
				<xsl:attribute name="append">true</xsl:attribute>
			</xsl:if>
			
			<xsl:element name="location">
				<xsl:attribute name="type">URL</xsl:attribute>
				<xsl:value-of select="jsdl:URI"/>
			</xsl:element>
			
		</xsl:element>
	</xsl:template>
	
	<xsl:template match="jsdl-posix:Environment">
		<xsl:element name="variable">
			<xsl:attribute name="name">
				<xsl:value-of select="@name"/>
			</xsl:attribute>
			<xsl:call-template name="FILESYSTEM_VALUE">
				<xsl:with-param name="FILESYSTEM_NAME">
					<xsl:value-of select="@filesystemName"/>
				</xsl:with-param>
				<xsl:with-param name="VALUE">
					<xsl:value-of select="."/>
				</xsl:with-param>
			</xsl:call-template>
		</xsl:element>
	</xsl:template>
	
<xsl:template name="FILESYSTEM_VALUE">
	<xsl:param name="FILESYSTEM_NAME"/>
	<xsl:param name="VALUE"/>
	<xsl:choose>
		<xsl:when test="$FILESYSTEM_NAME and $FILESYSTEM_NAME != ''">
			<xsl:variable name="FILESYSTEM"  select="key('FILESYSTEMS', string($FILESYSTEM_NAME))"/>
			<xsl:choose>
				<xsl:when test="$FILESYSTEM">
					<xsl:value-of select="$FILESYSTEM/jsdl:MountPoint"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:message terminate="no">
						Delcaration of following filesystem is missing: <xsl:value-of select="$FILESYSTEM_NAME"/>
					</xsl:message>
				</xsl:otherwise>
			</xsl:choose>
			<xsl:value-of select="$VALUE"/>
		</xsl:when>
		<xsl:otherwise>
			<xsl:if test="starts-with($VALUE, '/')">
				<xsl:value-of select="$FILE_URL_PREFIX"/>
			</xsl:if>
			<xsl:value-of select="$VALUE"/>
		</xsl:otherwise>
	</xsl:choose>
</xsl:template>

<xsl:template name="GET_FILE_NAME">
	<xsl:param name="FILE_PATH"/>
		
	<xsl:choose>
		<xsl:when test="not(contains($FILE_PATH,'/'))">
			<xsl:value-of select="$FILE_PATH"/>
		</xsl:when>
		<xsl:otherwise>
			<xsl:call-template name="GET_FILE_NAME">
				<xsl:with-param name="FILE_PATH">
					<xsl:value-of select="substring-after($FILE_PATH,'/')"/>
				</xsl:with-param>
			</xsl:call-template>
		</xsl:otherwise>
	</xsl:choose>
</xsl:template>

<xsl:template name="EXECUTION_TIME_TEMPLATE">
	<xsl:apply-templates select="//qcg:executionTime" mode="COPY_OF_MODE"/>
</xsl:template>

 <!-- =====================================================================================================================-->
        <xsl:template match="*" mode="COPY_OF_MODE">
                <xsl:choose>
                        <xsl:when test="node()">
                                <xsl:element name="{local-name()}">
                                        <xsl:copy-of select="@*"/>
                                        <xsl:apply-templates select="node()" mode="COPY_OF_MODE"/>
                                </xsl:element>
                        </xsl:when>
                        <xsl:otherwise>
                                <xsl:element name="{local-name()}">
                                        <xsl:copy-of select="@*"/>
                                </xsl:element>
                        </xsl:otherwise>
                </xsl:choose>
        </xsl:template>
 <!-- =====================================================================================================================-->

		
</xsl:stylesheet>
